@@ -618,10 +618,14 @@ kgaps_conf_int <- function(theta_mle, ss, conf = 95) {
   }
   ci_low <- 0
   ci_up <- 1
-  if (ss$N1 > 0) {
+  # If ob_fn(0) <= 0 then the log-likelihood does not fall to the cutoff in
+  # (0, theta_mle) so we set the lower limit of the confidence interval to 0
+  if (ss$N1 > 0 && ob_fn(0) > 0) {
     ci_low <- stats::uniroot(ob_fn, c(0, theta_mle))$root
   }
-  if (ss$N0 > 0) {
+  # If ob_fn(1) <= 0 then the log-likelihood does not fall to the cutoff in
+  # (theta_mle, 1) so we set the upper limit of the confidence interval to 1
+  if (ss$N0 > 0 && ob_fn(1) > 0) {
     ci_up <- stats::uniroot(ob_fn, c(theta_mle, 1))$root
   }
   return(c(ci_low, ci_up))
@@ -706,6 +710,144 @@ kgaps_imt_old <- function(data, u, k = 1) {
   return(res)
 }
 
+# ============================== kgaps_exp_info ============================= #
+#' @keywords internal
+#' @rdname exdex-internal
+kgaps_exp_info <- function(theta, ss, inc_cens) {
+  # How many observations are not right-censored?
+  # Note: if inc_cens = TRUE then ss$N1 and ss$n_dgaps are inflated by
+  # right-censored observations.  ss$n_dgaps is inflated by 1 for each
+  # right-censored observation and ss$N1 by 1/2.
+  # Subtract the excess from N0 + N1 to get the total number of observations
+  # that are not right-censored
+  not_right_censored <- ss$N0 + ss$N1 - (ss$n_kgaps - ss$N0 - ss$N1)
+  term1 <- 1 / (1 - theta)
+  term2 <- 2 / theta
+  # Add expected contributions from right-censored inter-exceedance times that
+  # are not left-censored
+  if (inc_cens) {
+    term3 <- 2 / theta
+  } else {
+    term3 <- 0
+  }
+  val <- not_right_censored * (term1 + term2) + term3
+  return(val)
+}
+
+# ============== Functions used by dgaps() and confint.dgaps() ============== #
+
+# =============================== dgaps_loglik ================================
+# The argument n_dgaps is not used here but it is included because it is
+# included in the list returned by dgaps_stat()
+#' @keywords internal
+#' @rdname exdex-internal
+dgaps_loglik <- function(theta, N0, N1, sum_qtd, n_dgaps, q_u, D) {
+#  if (theta < 0 || theta > 1) {
+#    return(-Inf)
+#  }
+  loglik <- 0
+  if (N1 > 0) {
+    loglik <- loglik + 2 * N1 * log(theta) - sum_qtd * theta
+  }
+  if (N0 > 0) {
+    loglik <- loglik + N0 * log(1 - theta * exp(-theta * q_u * D))
+  }
+  return(loglik)
+}
+
+# ========================= g(theta) and its derivatives ==================== #
+#' @keywords internal
+#' @rdname exdex-internal
+g_theta <- function(theta, q_u, D) {
+  d <- q_u * D
+  return(log(1 - theta * exp(-theta * d)))
+}
+
+#' @keywords internal
+#' @rdname exdex-internal
+gd_theta <- function(theta, q_u, D) {
+  d <- q_u * D
+  td <- theta * d
+  val <- (td - 1) / (exp(td) - theta)
+  return(val)
+}
+
+#' @keywords internal
+#' @rdname exdex-internal
+gdd_theta <- function(theta, q_u, D) {
+  d <- q_u * D
+  etd <- exp(theta * d)
+  val <- -(theta * d ^ 2 * etd - 2 * d * etd + 1) / (etd - theta) ^ 2
+  return(val)
+}
+
+#' @keywords internal
+#' @rdname exdex-internal
+gddd_theta <- function(theta, q_u, D) {
+  d <- q_u * D
+  etd <- exp(theta * d)
+  num <- theta * d ^ 3 * etd * (etd + theta) -
+    3 * d ^ 2 * etd * (etd + theta) + 6 * d * etd - 2
+  den <- (etd - theta) ^ 3
+  val <- num / den
+  return(val)
+}
+
+# ============================== dgaps_exp_info ============================= #
+#' @keywords internal
+#' @rdname exdex-internal
+dgaps_exp_info <- function(theta, ss, inc_cens) {
+  # How many observations are not right-censored?
+  # Note: if inc_cens = TRUE then ss$N1 and ss$n_dgaps are inflated by
+  # right-censored observations.  ss$n_dgaps is inflated by 1 for each
+  # right-censored observation and ss$N1 by 1/2.
+  # Subtract the excess from N0 + N1 to get the total number of observations
+  # that are not right-censored
+  not_right_censored <- ss$N0 + ss$N1 - (ss$n_dgaps - ss$N0 - ss$N1)
+  # Following eqn (11) on page 202 of Holesovsky and Fusek (2020)
+  d <- ss$q_u * ss$D
+  emtd <- exp(-theta * d)
+  term1 <- (theta * d ^ 2 - 2 * d + emtd) / (1 - theta * emtd)
+  term2 <- 2 / theta
+  # Add expected contributions from right-censored inter-exceedance times that
+  # are not left-censored
+  if (inc_cens) {
+    term3 <- 2 * emtd / theta
+  } else {
+    term3 <- 0
+  }
+  val <- not_right_censored * emtd * (term1 + term2) + term3
+  return(val)
+}
+
+# ============================== dgaps_conf_int ===============================
+#' @keywords internal
+#' @rdname exdex-internal
+dgaps_conf_int <- function(theta_mle, ss, conf = 95) {
+  cutoff <- stats::qchisq(conf / 100, df = 1)
+  theta_list <- c(list(theta = theta_mle), ss)
+  max_loglik <- do.call(dgaps_loglik, theta_list)
+  ob_fn <- function(theta) {
+    theta_list$theta <- theta
+    loglik <- do.call(dgaps_loglik, theta_list)
+    return(2 * (max_loglik - loglik) - cutoff)
+  }
+  x <- seq(0.001, 0.999, len = 100)
+  ci_low <- 0
+  ci_up <- 1
+  # If ob_fn(0) <= 0 then the log-likelihood does not fall to the cutoff in
+  # (0, theta_mle) so we set the lower limit of the confidence interval to 0
+  if (ss$N1 > 0 && ob_fn(0) > 0) {
+    ci_low <- stats::uniroot(ob_fn, c(0, theta_mle))$root
+  }
+  # If ob_fn(1) <= 0 then the log-likelihood does not fall to the cutoff in
+  # (theta_mle, 1) so we set the upper limit of the confidence interval to 1
+  if (ss$N0 > 0 && ob_fn(1) > 0) {
+    ci_up <- stats::uniroot(ob_fn, c(theta_mle, 1))$root
+  }
+  return(c(ci_low, ci_up))
+}
+
 # ========================= Function used by iwls() ========================= #
 
 # ================================== iwls_fun =================================
