@@ -148,3 +148,25 @@ test_that("kgaps lik intervals don't depend on conf_scale", {
   testthat::expect_identical(res1$cis["lik", ], res2$cis["lik", ])
 })
 
+# ================================== dgaps ===================================
+
+#context("confint.dgaps")
+
+u <- quantile(newlyn, probs = 0.90)
+
+res <- dgaps(newlyn, u)
+res1 <- confint(res)
+res2 <- confint(res, conf_scale = "log")
+test_that("dgaps lik intervals don't depend on conf_scale", {
+  testthat::expect_identical(res1$cis["lik", ], res2$cis["lik", ])
+})
+
+# Repeat for inc_cens = TRUE
+
+res <- dgaps(newlyn, u, inc_cens = TRUE)
+res1 <- confint(res)
+res2 <- confint(res, conf_scale = "log")
+test_that("dgaps lik intervals don't depend on conf_scale", {
+  testthat::expect_identical(res1$cis["lik", ], res2$cis["lik", ])
+})
+
