@@ -12,7 +12,7 @@ res1 <- signif(c(theta$theta, theta$se),
 res2 <- summary(theta)$matrix
 
 test_that("Fitted object and summary() agree", {
-  testthat::expect_equivalent(res1, res2)
+  testthat::expect_equal(res1, res2, ignore_attr = TRUE)
 })
 
 # Check that kgaps() gives the same output for vector and matrix data input
@@ -23,5 +23,5 @@ theta$call <- NULL
 theta2$call <- NULL
 
 test_that("kgaps: vector data vs matrix data", {
-  testthat::expect_equivalent(theta, theta2)
+  testthat::expect_equal(theta, theta2, ignore_attr = TRUE)
 })
